@@ -15,7 +15,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: branchman.c,v 1.47 2005/08/26 19:42:20 cwright Exp $
+ *  $Id: branchman.c,v 1.51 2005/10/20 16:24:41 cwright Exp $
  */
 
 #include "fist.h"
@@ -159,7 +159,7 @@ int unionfs_ioctl_incgen(struct file *file, unsigned int cmd, unsigned long arg)
 
 	sb = file->f_dentry->d_sb;
 
-	lock_super(sb);
+	unionfs_write_lock(sb);
 	if ((err = newputmap(sb)))
 		goto out;
 
@@ -169,10 +169,9 @@ int unionfs_ioctl_incgen(struct file *file, unsigned int cmd, unsigned long arg)
 	atomic_set(&dtopd(sb->s_root)->udi_generation, err);
 	atomic_set(&itopd(sb->s_root->d_inode)->uii_generation, err);
 
-	print_exit_status(err);
-
       out:
-	unlock_super(sb);
+	unionfs_write_unlock(sb);
+	print_exit_status(err);
 	return err;
 }
 
@@ -222,14 +221,8 @@ int unionfs_ioctl_addbranch(struct inode *inode, unsigned int cmd,
 	if (!(path = getname(addargs->ab_path)))
 		goto out;
 
-//DQ: 2.6 has a different way of doing this
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-	/* Look it up */
-	if (path_init(path, LOOKUP_FOLLOW, &nd))
-		err = path_walk(path, &nd);
-#else
 	err = path_lookup(path, LOOKUP_FOLLOW, &nd);
-#endif
+
 	RECORD_PATH_LOOKUP(&nd);
 	if (err)
 		goto out;
@@ -239,7 +232,7 @@ int unionfs_ioctl_addbranch(struct inode *inode, unsigned int cmd,
 		goto out;
 	}
 
-	lock_super(inode->i_sb);
+	unionfs_write_lock(inode->i_sb);
 	lock_dentry(inode->i_sb->s_root);
 
 	err = -EINVAL;
@@ -379,7 +372,7 @@ int unionfs_ioctl_addbranch(struct inode *inode, unsigned int cmd,
 
       out:
 	unlock_dentry(inode->i_sb->s_root);
-	unlock_super(inode->i_sb);
+	unionfs_write_unlock(inode->i_sb);
 
 	KFREE(new_hidden_mnt);
 	KFREE(new_udi_dentry);
@@ -490,7 +483,7 @@ int unionfs_ioctl_rdwrbranch(struct inode *inode, unsigned int cmd,
 
 	print_entry_location();
 
-	lock_super(inode->i_sb);
+	unionfs_write_lock(inode->i_sb);
 	lock_dentry(inode->i_sb->s_root);
 
 	if ((err = newputmap(inode->i_sb)))
@@ -525,8 +518,8 @@ int unionfs_ioctl_rdwrbranch(struct inode *inode, unsigned int cmd,
 	err = 0;
 
       out:
-	unlock_super(inode->i_sb);
 	unlock_dentry(inode->i_sb->s_root);
+	unionfs_write_unlock(inode->i_sb);
 	KFREE(rdwrargs);
 
 	print_exit_status(err);
