@@ -15,7 +15,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: unionfs.h,v 1.157 2005/09/12 21:14:22 dquigley Exp $
+ *  $Id: unionfs.h,v 1.163 2005/10/24 18:52:17 cwright Exp $
  */
 
 #ifndef __UNIONFS_H_
@@ -23,19 +23,17 @@
 
 #ifdef __KERNEL__
 
-#if (!defined(UNIONFS_UNSUPPORTED)) &&(LINUX_VERSION_CODE < KERNEL_VERSION(2,4,20)) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9))
+#if (!defined(UNIONFS_UNSUPPORTED)) && (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,9))
 #warning You are compiling Unionfs on an unsupported kernel version.
 #warning To compile Unionfs, you will need to define UNIONFS_UNSUPPORTED.
 #warning Try adding: EXTRACFLAGS=-DUNIONFS_UNSUPPORTED to fistdev.mk
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 #include <linux/spinlock.h>
 #define wq_write_lock_irqsave(lock,flags) spin_lock_irqsave(lock,flags)
 #define wq_write_unlock(lock) spin_unlock(lock)
 #define wq_write_lock_irq(lock) spin_lock_irq(lock)
 #define wq_write_unlock_irqrestore(lock,flags) spin_unlock_irqrestore(lock,flags)
-#endif
 
 /* number of characters while generating unique temporary file names */
 #define	UNIONFS_TMPNAM_LEN	12
@@ -91,12 +89,11 @@ struct unionfs_inode_info {
 	/* to keep track of reads/writes for unlinks before closes */
 	atomic_t uii_totalopens;
 };
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+
 struct unionfs_inode_container {
 	struct unionfs_inode_info info;
 	struct inode vfs_inode;
 };
-#endif
 
 /* unionfs dentry data in memory */
 struct unionfs_dentry_info {
@@ -130,6 +127,7 @@ struct unionfs_sb_info {
 
 	atomic_t usi_generation;
 	unsigned long usi_mount_flag;
+	struct rw_semaphore usi_rwsem;
 
 	int usi_persistent;
 
@@ -241,30 +239,21 @@ struct unionfs_file_info {
 #define fbend(file) (ftopd(file)->b_end)
 
 /* Inode to private data */
-#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,0)
-#define itopd(ino) ((struct unionfs_inode_info *)(ino)->u.generic_ip)
-#define itopd_lhs(ino) ((ino)->u.generic_ip)
-#else
 static inline struct unionfs_inode_info *itopd(const struct inode *inode)
 {
 	return
 	    &(container_of(inode, struct unionfs_inode_container, vfs_inode)->
 	      info);
 }
-#endif
+
 #define itohi_ptr(ino) (itopd(ino)->uii_inode_p)
 #define itohi_inline(ino) (itopd(ino)->uii_inode_i)
 #define ibstart(ino) (itopd(ino)->b_start)
 #define ibend(ino) (itopd(ino)->b_end)
 
 /* Superblock to private data */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-#define stopd(super) ((struct unionfs_sb_info *)(super)->u.generic_sbp)
-#define stopd_lhs(super) ((super)->u.generic_sbp)
-#else
 #define stopd(super) ((struct unionfs_sb_info *)(super)->s_fs_info)
 #define stopd_lhs(super) ((super)->s_fs_info)
-#endif
 #define sbstart(sb) 0
 #define sbend(sb) stopd(sb)->b_end
 #define sbmax(sb) (stopd(sb)->b_end + 1)
@@ -273,6 +262,11 @@ static inline struct unionfs_inode_info *itopd(const struct inode *inode)
 #define stohiddenmnt_ptr(super) (stopd(super)->usi_hidden_mnt_p)
 #define stohiddenmnt_inline(super) (stopd(super)->usi_hidden_mnt_i)
 
+#define unionfs_read_lock(sb) down_read(&stopd(sb)->usi_rwsem)
+#define unionfs_read_unlock(sb) up_read(&stopd(sb)->usi_rwsem)
+#define unionfs_write_lock(sb) down_write(&stopd(sb)->usi_rwsem)
+#define unionfs_write_unlock(sb) up_write(&stopd(sb)->usi_rwsem)
+
 /* The UNIONFS_NDEBUG versions are defines, the debug versions are inline
  * functions with extra checks. */
 #ifdef UNIONFS_NDEBUG
@@ -281,7 +275,7 @@ static inline struct unionfs_inode_info *itopd(const struct inode *inode)
 #include "unionfs_debugmacros.h"
 #endif
 
-/* The double lock function needs to go after the debugmacros, so that 
+/* The double lock function needs to go after the debugmacros, so that
  * dtopd is defined.  */
 static inline void double_lock_dentry(struct dentry *d1, struct dentry *d2)
 {
@@ -318,16 +312,18 @@ extern int create_whiteout_parent(struct dentry *parent_dentry,
 				  const char *filename, int start);
 /* copies a file from dbstart to newbindex branch */
 extern int copyup_file(struct inode *dir, struct file *file, int bstart,
-		       int newbindex, int size);
+		       int newbindex, loff_t size);
 extern int copyup_named_file(struct inode *dir, struct file *file,
-			     char *name, int bstart, int new_bindex, int len);
+			     char *name, int bstart, int new_bindex,
+			     loff_t len);
 
 /* copies a dentry from dbstart to newbindex branch */
 extern int copyup_dentry(struct inode *dir, struct dentry *dentry, int bstart,
-			 int new_bindex, struct file **copyup_file, int len);
+			 int new_bindex, struct file **copyup_file, loff_t len);
 extern int copyup_named_dentry(struct inode *dir, struct dentry *dentry,
-			       int bstart, int new_bindex, char *name,
-			       int namelen, struct file **copyup_file, int len);
+			       int bstart, int new_bindex, const char *name,
+			       int namelen, struct file **copyup_file,
+			       loff_t len);
 
 extern int remove_whiteouts(struct dentry *dentry, struct dentry *hidden_dentry,
 			    int bindex);
@@ -368,11 +364,7 @@ extern int unionfs_rename(struct inode *old_dir, struct dentry *old_dentry,
 int unionfs_unlink(struct inode *dir, struct dentry *dentry);
 int unionfs_rmdir(struct inode *dir, struct dentry *dentry);
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
 int unionfs_d_revalidate(struct dentry *dentry, struct nameidata *nd);
-#else
-int unionfs_d_revalidate(struct dentry *dentry, int flags);
-#endif
 
 /* The values for unionfs_interpose's flag. */
 #define INTERPOSE_DEFAULT	0
@@ -400,25 +392,21 @@ int unionfs_ioctl_queryfile(struct file *file, unsigned int cmd,
 /* Verify that a branch is valid. */
 int check_branch(struct nameidata *nd);
 
-#if defined(UNIONFS_XATTR) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20)
+#ifdef UNIONFS_XATTR
 /* Extended attribute functions. */
 extern void *xattr_alloc(size_t size, size_t limit);
 extern void xattr_free(void *ptr, size_t size);
 
-extern int unionfs_getxattr(struct dentry *dentry, const char *name,
-			    void *value, size_t size);
+extern ssize_t unionfs_getxattr(struct dentry *dentry, const char *name,
+				void *value, size_t size);
 extern int unionfs_removexattr(struct dentry *dentry, const char *name);
-extern int unionfs_listxattr(struct dentry *dentry, char *list, size_t size);
+extern ssize_t unionfs_listxattr(struct dentry *dentry, char *list,
+				 size_t size);
 
-#if defined(FIST_SETXATTR_CONSTVOID) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
 int unionfs_setxattr(struct dentry *dentry, const char *name, const void *value,
 		     size_t size, int flags);
-#else
-int unionfs_setxattr(struct dentry *dentry, const char *name, void *value,
-		     size_t size, int flags);
-#endif
 
-#endif				/* LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,20) */
+#endif				/* UNIONFS_XATTR */
 
 #define copy_inode_size(dst, src) \
     dst->i_size = src->i_size; \
@@ -446,9 +434,8 @@ static inline int get_nlinks(struct inode *inode)
 		hidden_inode = itohi_index(inode, bindex);
 		if (!hidden_inode || !S_ISDIR(hidden_inode->i_mode))
 			continue;
-#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
 		ASSERT(hidden_inode->i_nlink >= 0);
-#endif
+
 		/* A deleted directory. */
 		if (hidden_inode->i_nlink == 0)
 			continue;
@@ -514,11 +501,7 @@ static inline void fist_copy_attr_all(struct inode *dest,
 
 	//DQ: This was a change I noticed in the templates. In 2.6 they removedi_attr_flags.
 	//Which makes me think they rolled it into flags.
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
-	dest->i_attr_flags = src->i_attr_flags;
-#else
 	dest->i_flags = src->i_flags;
-#endif
 
 	print_exit_location();
 }
